#include "../nanika/len_string.h"

#include <iomanip>
#include <iostream>
#include <string>
#include <vector>

using namespace nanika;
using namespace std;

// Checks a given expression is true or not.
#define CHECK(expr) cout << (expr) << ": " << #expr << endl;

int main()
{
	// Uses "true" or "false" instead of "1" or "0".
	cout.setf(ios::boolalpha);

	// Checks empty strings.
	len_string empty;
	CHECK(empty.empty());
	CHECK(empty == "");
	empty.assign("empty", 0);
	CHECK(empty.empty());

	// Checks comparison operators for C strings.
	len_string first = "first";
	CHECK(first == "first");
	CHECK(first != "second");
	CHECK(first > "First");
	CHECK(first < "second");

	// Checks simple comparison functions of nanika::len_string.
	len_string second("second", 6);
	CHECK(!second.compare("second"));

	// Checks comparison operators for nanika::len_string.
	CHECK(first == first);
	CHECK(first != second);
	CHECK(first < second);
	CHECK(first > empty);

	// Finds C strings from a string.
	CHECK(first.find("first") == 0);
	CHECK(first.find("second") == len_string::npos);
	CHECK(first.find("irs") == 1);

	// Finds specified characters from a string.
	CHECK(first.find_first_of('f') == 0);
	CHECK(first.find_first_of("pqr") == 2);
	CHECK(first.find_first_of("xyz") == len_string::npos);

	// Removes white-spaces from a string.
	len_string space;
	space = "  space  ";
	CHECK(space.trim_start() == "space  ");
	CHECK(space.trim_end() == "  space");
	CHECK(space.trim() == "space");

	// Finds characters and C strings from a string.
	len_string text = "This is a pen.";
	CHECK(text.find("is") == 2);
	CHECK(text.find_first_of('i') == 2);
	CHECK(text.find_last_of('i') == 5);
	CHECK(text.rfind("is") == 5);
	CHECK(text.find_first_not_of("this", len_string::lower()) == 4);
	CHECK(text.find_last_not_of(" APEN.", len_string::lower()) == 6);

	// Splits a string into parts.
	len_string url = "http://localhost/wiki/index.php";
	vector<len_string> parts;
	url.split(parts, "/", true);
	CHECK(parts.size() == 4);
	CHECK(parts[0] == "http:");
	CHECK(parts[1] == "localhost");
	CHECK(parts[2] == "wiki");
	CHECK(parts[3] == "index.php");

	return 0;
}
