#include "../nanika/raw_string.h"

#include <iomanip>
#include <iostream>
#include <string>

using namespace nanika;
using namespace std;

// Checks a given expression is true or not.
#define CHECK(expr) cout << (expr) << ": " << #expr << endl;

int main()
{
	// Uses "true" or "false" instead of "1" or "0".
	cout.setf(ios::boolalpha);

	// Checks empty strings.
	raw_string empty;
	CHECK(!strcmp(empty.c_str(), ""));
	CHECK(empty.empty());
	CHECK(raw_string("").empty());

	// Generates a simple string and checks it as a C string.
	raw_string first("first");
	CHECK(!strcmp(first.c_str(), "first"));

	// Copies a string and compares them as C strings.
	raw_string first_copy(first);
	CHECK(!strcmp(first_copy.c_str(), "first"));

	// Gets a suffix of a string and checks if it is correct or not.
	CHECK(raw_string(first.c_str(), 2) == "rst");

	// Checks a suffix by using an operator of nanika::len_string.
	first_copy.assign(first, 3);
	CHECK(first_copy == "st");

	// Generates another string and checks an equal operator.
	raw_string second("second");
	CHECK(second == "second");

	// Checks comparison operators for nanika::len_string.
	CHECK(first < second);
	CHECK(first > empty);
	CHECK(first == first);
	CHECK(first != second);

	// Checks comparison operators for C strings.
	CHECK(first < "second");
	CHECK(first > "");
	CHECK(first == "first");
	CHECK(first != "second");

	// Checks simple comparison functions of nanika::len_string.
	CHECK(first.compare(second) < 0);
	CHECK(first.compare("the first", 4) == 0);
	CHECK(first.compare("fire", 0, 3) == 0);
	CHECK(first.substr(3).compare("st") == 0);
	CHECK(first.substr(3).compare("roast", 3) == 0);
	CHECK(first.substr(3).compare("roast beaf", 3, 2) == 0);

	// Checks advanced comparison functions of nanika::len_string.
	CHECK(first.compare(first, raw_string::as_is()) == 0);
	CHECK(first.compare("the FIRST", 4, raw_string::lower()) == 0);
	CHECK(first.compare("Fire", 0, 3, raw_string::upper()) == 0);
	CHECK(first.substr(3).compare("ST", raw_string::lower()) == 0);
	CHECK(first.substr(3).compare("ROAST", 3, raw_string::upper()) == 0);
	CHECK(first.substr(3).compare("ROAST", 3, 2, raw_string::as_is()) != 0);

	// Checks global comparison operators.
	CHECK("" < first);
	CHECK("first" <= first);
	CHECK("second" > first);
	CHECK("first" >= first);
	CHECK("first" == first);
	CHECK("second" != first);

	// Checks a function to generate suffix strings.
	CHECK(first.substr(1) == "irst");
	CHECK(first.substr(5) == "");

	// Finds C strings from a string.
	CHECK(first.find("irs") != raw_string::npos);
	CHECK(first.find("start") == raw_string::npos);
	CHECK(first.find('f') != raw_string::npos);
	CHECK(first.find('x') == raw_string::npos);

	// Finds specified characters from a string.
	CHECK(first.find_first_of("opqr") == 2);
	CHECK(first.find_first_of("xyz") == raw_string::npos);
	CHECK(first.find_first_of('f') == 0);
	CHECK(first.find_first_of('x') == raw_string::npos);

	// Finds unspecified characters from a string.
	CHECK(first.find_first_not_of("fit") == 2);
	CHECK(first.find_first_not_of("first") == raw_string::npos);
	CHECK(first.find_first_not_of('z') == 0);
	CHECK(first.find_first_not_of('f') == 1);

	// Writes out a string to a buffer.
	char buf[16] = "";
	CHECK(first.copy(buf, 100) == 5);
	CHECK(first == buf);
	CHECK(second.copy(buf, 3) == 3);
	CHECK(raw_string("secst") == buf);
	CHECK(first.copy(buf, 3, raw_string::upper()) == 3);
	CHECK(raw_string("FIRst") == buf);

	// Swap strings.
	first.swap(second);
	CHECK((first == "second") && (second == "first"));
	second.swap(first);
	CHECK((first == "first") && (second == "second"));

	return 0;
}
