#include "../nanika/string_filter.h"

#include <ctime>

#include <fstream>
#include <iostream>
#include <string>
#include <vector>

using namespace nanika;
using namespace std;

// Checks a given expression and if false, this macro throws an exception.
#define IS_TRUE(expr) if (!(expr)) throw "Error: " #expr
// Checks a given expression is true or not.
#define CHECK(expr) cout << (expr) << ": " << #expr << endl;

namespace
{
	// Prints messages in the constructor and the destructor.
	class Timer
	{
	public:
		// Initializes a message and a time.
		// Then, prints a message.
		Timer(const char *msg) : msg_(msg), cl_(clock())
		{
			cout << "Begin: " << msg_ << endl;
		}
		// Prints a message with the time passed after construction.
		~Timer()
		{
			cout << "End: " << msg_ << ": "
				<< (1.0 * (clock() - cl_) / CLOCKS_PER_SEC) << " sec" << endl;
		}

	private:
		// Message.
		const string msg_;
		// Constructed time.
		const clock_t cl_;
	};
}

int main(int argc, char *argv[])
{
	// Checks the number of arguments.
	if (argc < 2)
	{
		cerr << "Usage: " << argv[0] << " Keyset Texts..." << endl;
		return 1;
	}

	// Constructs a filter dictionary.
	string_filter filter;
	{
		Timer timer("Filter construction");
		filter = string_filter(argv[1]);
	}

	// The filter dictionary must be valid.
	IS_TRUE(filter.is_valid());
	// Then, prints the dictionary size.
	cout << "Filter size: " << filter.size() << " bytes" << endl;

	// Tests text files specified as command line arguments.
	for (int i = 2; i < argc; ++i)
	{
		// Opens a text file.
		cout << "Text file: " << argv[i] << endl;
		ifstream text(argv[i]);
		IS_TRUE(!text.fail());

		// Initializes a timer.
		Timer timer("Matching");

		int count = 0;
		string line;

		// Reads lines from a text file.
		while (getline(text, line))
		{
			if (!filter.find(line.c_str()).empty())
				++count;
		}

/*
		// Reads lines from a text file.
		vector<string_filter::result_pair> results;
		while (getline(text, line))
		{
			// Splits each line and counts the number of parts.
			len_string line_str(line.c_str(), line.length());
			count += filter.split(line_str, results);
			results.clear();
		}
*/

/*
		// Reads lines from a text file.
		vector<len_string> results;
		while (getline(text, line))
		{
			// Finds all substring keys.
			len_string line_str(line.c_str(), line.length());
			count += filter.find_all(line_str, results);
			results.clear();
		}
*/

		cout << "Count: " << count << endl;
	}

	return 0;
}
